package br.usp.poli.java.ihc;
/*
Copyright (C) 2008 Fbio Levy Siqueira (fabiolevy@yahoo.com.br)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
import java.awt.Color;
import java.awt.Dimension;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.InvalidPropertiesFormatException;
import java.util.Properties;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import br.usp.poli.java.IJogadorManilhaFixa;
import br.usp.poli.java.IJogadorManilhaVariavel;

/**
 * Interface homem mquina do jogo de truco usando Swing.<br>
 * Exemplo de uso:<br>
 * <code>
 * Truco truco = new Truco(new JogoManilhaFixa(), new JogoManilhaVariavel(), JogadorSimples.class);
 * truco.iniciar(); // cria a tela e comea o jogo
 * </code>
 * @author FLevy
 * @version 1.0
 */

@SuppressWarnings("serial")
public final class Truco extends JFrame{
	private JogoDeTruco manilhaFixa;
	private JogoDeTruco manilhaVariavel;
	private Class<? extends IJogadorManilhaFixa> computadorManilhaFixa;
	private Class<? extends IJogadorManilhaVariavel> computadorManilhaVariavel;
	private Menu menu;
	private Mesa mesa;
	private static Properties propriedades = null;
	private static final String ARQUIVO = "/truco.properties";
	
	/**
	 * Cria um jogo de truco a partir de duas implementaes de JogoDeTruco,
	 * uma para manilha fixa e outra para manilha varivel.<br>
	 * @param manilhaFixa A implementao para manilha fixa.
	 * @param manilhaVariavel A implementao para manilha varivel.
	 * @param classeJogadorComputador Uma classe que implementa um jogador automatizado.
	 * Em Java  possvel obter um objeto com informaes de uma classe (usando
	 * Reflection). Esse objeto est disponvel como um atributo de classe (static)
	 * "class". Exemplo: Truco.class retorna um objeto com as informaes da classe
	 * desta classe (Truco).
	 */
	public Truco(JogoDeTruco manilhaFixa, JogoDeTruco manilhaVariavel,
			Class<? extends IJogadorManilhaFixa> classeJogadorComputadorManilhaFixa,
			Class<? extends IJogadorManilhaVariavel> classeJogadorComputadorManilhaVariavel) {
		super("Jogo de Truco");
		this.manilhaFixa = manilhaFixa;
		this.manilhaVariavel = manilhaVariavel;
		this.computadorManilhaFixa = classeJogadorComputadorManilhaFixa;
		this.computadorManilhaVariavel = classeJogadorComputadorManilhaVariavel;
	}
	
	public void iniciar() {
		this.getContentPane().setBackground(obterCor("fundo"));
		
		this.menu = new Menu(this);
		this.setJMenuBar(menu);
		
		menu.iniciar();
		
		try {
			this.setMinimumSize(new Dimension(Integer.parseInt(obterPropriedade("largura_tela")), Integer.parseInt(obterPropriedade("altura_tela"))));
		} catch (NumberFormatException e) {
			throw new ProblemaInterface("Erro ao obter as propriedades do arquivo de configurao.");
		}
		this.setResizable(false);
		this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		this.setVisible(true);
	}
	
	public void dispose() {
		this.abortar();
		super.dispose();
		System.exit(0);
	}
	
	void abortar() {
		if (mesa == null) return;
		this.mesa.terminar();
	}
	
	public JogoDeTruco getJogoManilhaFixa() {
		return manilhaFixa;
	}
	
	public JogoDeTruco getJogoManilhaVariavel() {
		return manilhaVariavel;
	}
	
	public Class<? extends IJogadorManilhaFixa> getJogadorComputadorManilhaFixa() {
		return computadorManilhaFixa;
	}
	
	public Class<? extends IJogadorManilhaVariavel> getJogadorComputadorManilhaVariavel() {
		return computadorManilhaVariavel;
	}
	
	void comecarJogo(boolean jogoManilhaFixa, String nomeJ1, boolean j1Automatico,
			String nomeJ2, boolean j2Automatico) {
		JogadorIHC j1, j2;
		JogoDeTruco jogo;
		
		if (jogoManilhaFixa) jogo = manilhaFixa;
		else jogo = manilhaVariavel;
		
		// Criando os jogadores
		try {
			if (j1Automatico && jogoManilhaFixa)
				j1 = new JogadorAutomaticoManilhaFixa(computadorManilhaFixa.newInstance(), nomeJ1, true);
			else if (jogoManilhaFixa)
				j1 = new JogadorHumanoManilhaFixa(nomeJ1, true);
			else if (j1Automatico)
				j1 = new JogadorAutomaticoManilhaVariavel(computadorManilhaVariavel.newInstance(), nomeJ1, true);
			else
				j1 = new JogadorHumanoManilhaVariavel(nomeJ1, true);
			
			if (j2Automatico && jogoManilhaFixa)
				j2 = new JogadorAutomaticoManilhaFixa(computadorManilhaFixa.newInstance(), nomeJ2, true);
			else if (jogoManilhaFixa)
				j2 = new JogadorHumanoManilhaFixa(nomeJ2, true);
			else if (j2Automatico)
				j2 = new JogadorAutomaticoManilhaVariavel(computadorManilhaVariavel.newInstance(), nomeJ2, true);
			else
				j2 = new JogadorHumanoManilhaVariavel(nomeJ2, true);
		} catch (Exception e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(this,
				    "No foi possvel criar um jogador.",
				    "Erro",
				    JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		if (mesa != null) this.remove(mesa);
		mesa = new Mesa(this, j1, j2, jogoManilhaFixa, jogo);
		this.add(mesa);
		
		mesa.iniciar();
		//this.pack();
		//System.out.println(this.getSize());
	}
	
	void fimDoJogo() {
		menu.reiniciar();
	}
	
	private static void carregarPropriedades() {
		propriedades = new Properties();
		try {
			propriedades.loadFromXML(Truco.class.getResourceAsStream(ARQUIVO));
		} catch (InvalidPropertiesFormatException e) {
			throw new ProblemaInterface("Formato incorreto do arquivo de propriedades.");
		} catch (FileNotFoundException e) {
			throw new ProblemaInterface("Arquivo de propriedades no encontrado (" + ARQUIVO + ").");
		} catch (IOException e) {
			throw new ProblemaInterface("Erro de I/O ao carregar o arquivo de configurao.");
		}
	}
	
	static String obterPropriedade(String propriedade) {
		if (propriedades == null) carregarPropriedades();
		return propriedades.getProperty(propriedade);
	}
	
	static Color obterCor(String propriedade) {
		try {
			return new Color(Integer.parseInt(obterPropriedade(propriedade), 16));
		} catch (NumberFormatException e) {
			throw new ProblemaInterface("Erro ao obter as propriedades do arquivo de configurao.");
		}
	}
}
